@@ -27,22 +27,7 @@
 #elif HAVE_EXT_HASH_MAP
 # include <ext/hash_map>
 #endif
-
-// this takes an IPv4 address (an in_addr struct) and returns it as a
-// string in dotted quad form.
-// dont forget to free what this returns
-char * in_addr2ascii(in_addr addr)
-{
-	char *ascii = (char *) malloc(16); // 12 for octets, 3 dots, 1 null
-	unsigned int iaddr = ntohl(addr.s_addr);
-
-	int oc1 = (iaddr & 0xFF000000)/16777216;
-	int oc2 = (iaddr & 0x00FF0000)/65536;
-	int oc3 = (iaddr & 0x0000FF00)/256;
-	int oc4 = (iaddr & 0x000000FF);
-	sprintf(ascii,"%d.%d.%d.%d",oc1,oc2,oc3,oc4);
-	return ascii;
-}
+#include <string.h>
 
 /*
  * getnlp takes a raw packet (with link level header) sniffed from an 
@@ -55,39 +40,50 @@ char * in_addr2ascii(in_addr addr)
 struct nlp *getnlp( const u_char *p, int dlt, const pcap_pkthdr *pcap )
 {
 	struct nlp *n = (struct nlp *) malloc( sizeof(struct nlp) );
-	
+
 	n->p=NULL;
-	n->proto=NLP_UNKNOWN;
 	n->ts = pcap->ts;
 	n->len = 0;
 	int vlan_frame = 0;
-	
+
 	if( dlt==DLT_EN10MB )
 	{
 		if( pcap->caplen < ENET_HEADER_LEN+IP_HEADER_LEN )
-			return n;
-		
+		{
+			free(n);
+			return NULL;
+		}
+
 		const struct sniff_ethernet *ethernet;
 		ethernet = (struct sniff_ethernet*)(p);
-		vlan_frame = (ntohs(ethernet->ether_type) == 0x8100);
-		if( ntohs(ethernet->ether_type) == 0x0800 || vlan_frame &&
-		    ntohs(*((uint16_t *)(p + ENET_HEADER_LEN + VLAN_HEADER_LEN - 2))) == 0x0800) // TODO: is this defined somewhere?
+		vlan_frame = ( ntohs(ethernet->ether_type) == ETHERTYPE_VLAN );
+		uint16_t ether_type;
+
+		if( vlan_frame )
+			ether_type = ntohs(*((uint16_t *)(p + ENET_HEADER_LEN + VLAN_HEADER_LEN - 2)));
+		else
+			ether_type = ntohs(ethernet->ether_type);
+
+		if( ether_type == ETHERTYPE_IP || ether_type == ETHERTYPE_IPV6 )
 		{
-			n->proto = NLP_IPV4;
 			n->len = pcap->caplen-ENET_HEADER_LEN-(vlan_frame ? VLAN_HEADER_LEN : 0);
 			n->p = (u_char *) malloc( sizeof(u_char) * n->len );
 			memcpy( (void *)n->p, (void *)(p+ENET_HEADER_LEN+(vlan_frame ? VLAN_HEADER_LEN : 0)), n->len);
 		}
 		else
-			return n;
+		{
+			free(n);
+			return NULL;
+		}
 	}
 	else if( dlt==DLT_LINUX_SLL )
 	{
 		if( pcap->caplen < SLL_HEADER_LEN+IP_HEADER_LEN )
-			return n;
+		{
+			free(n);
+			return NULL;
+		}
 
-		n->proto=NLP_IPV4;
-		//cerr << "pcap->caplen=" << pcap->caplen << endl;
 		n->len = pcap->caplen-SLL_HEADER_LEN;
 		n->p = (u_char *) malloc( sizeof(u_char) * n->len );
 		memcpy( (void *)n->p, (void *)(p+SLL_HEADER_LEN), n->len);
@@ -95,9 +91,11 @@ struct nlp *getnlp( const u_char *p, int dlt, const pcap_pkthdr *pcap )
 	else if( dlt==DLT_RAW || dlt==DLT_NULL )
 	{
 		if( pcap->caplen < IP_HEADER_LEN )
-			return n;
+		{
+			free(n);
+			return NULL;
+		}
 
-		n->proto=NLP_IPV4;
 		n->len = pcap->caplen;
 		n->p = (u_char *) malloc( sizeof(u_char) * n->len );
 		memcpy( (void *)n->p, (void *)(p), n->len);
@@ -116,29 +114,68 @@ struct nlp *getnlp( const u_char *p, int dlt, const pcap_pkthdr *pcap )
 // TODO: this should be split up into smaller functions
 bool checknlp( struct nlp *n )
 {
-	if( n->proto != NLP_IPV4 ) // IPv6 not yet supported
-		return false;
-	
+	struct sniff_ip *ip = (struct sniff_ip *)n->p;
+
+	//TODO: Improve this / split in two functions (IPv6/IPv4)
+	if( ip->ip_v == 6 )
+	{
+		if( n->len < IP6_HEADER_LEN + TCP_HEADER_LEN )
+		{
+			return false;
+		}
+
+		struct sniff_ip6 *ip6 = (struct sniff_ip6 *)n->p;
+
+		if( ip6->ip_next != IPPROTO_TCP ) 
+		{
+			return false;
+		}
+
+		struct sniff_tcp *tcp = (struct sniff_tcp *) (n->p + IP6_HEADER_LEN);
+
+		if( tcp->th_off < 5 ) // tcp header is at least 20 bytes long.
+		{
+			return false;
+		}
+
+		if( tcp->th_sport == 0 )
+			return false;
+
+		if( tcp->th_dport == 0 )
+			return false;
+
+		return true;
+	}
+
+	unsigned int ip_header_len = ip->ip_hl * 4;
+
 	// not enough data to do anything with this...
 	// we're only interested in IPv4 TCP Packets
-	if( n->len < IP_HEADER_LEN+TCP_HEADER_LEN )
+	if( n->len < ip_header_len + TCP_HEADER_LEN )
 		return false;
 	
-	struct sniff_ip *ip = (struct sniff_ip *)n->p;
-	
-	if( ntohs(ip->ip_len) < IP_HEADER_LEN+TCP_HEADER_LEN ) 
+	if( ip->ip_v != 4 )
+		return false;
+
+	if( ntohs(ip->ip_len) < ip_header_len + TCP_HEADER_LEN ) 
 		return false;
 
 	if( ip->ip_hl < 5 ) 
 		return false;
-	
+
 	if( ip->ip_p != IPPROTO_TCP ) 
 		return false;
 
-	struct sniff_tcp *tcp = (struct sniff_tcp *) (n->p+IP_HEADER_LEN);
-	
+	struct sniff_tcp *tcp = (struct sniff_tcp *) (n->p + ip_header_len);
+
 	if( tcp->th_off < 5 ) // tcp header is at least 20 bytes long.
 		return false;
+		
+	if( tcp->th_sport == 0 )
+		return false;
+
+	if( tcp->th_dport == 0 )
+		return false;
 
 	return true;
 }
